import requests as r

class Locations():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers

    def get(self, limit: int=50, offset: int=None, search: str=None, sort: str=None, order: str=None):
        """Get locations

        Args:
            limit (int, optional): Number of entries to return. Defaults to 50.
            offset (int, optional): Offset to use. Defaults to None.
            search (str, optional): A text string to search the locations data for. Defaults to None.
            sort (str, optional): Column to sort by. Defaults to None.
            order (str, optional): Order of sort 'asc' or 'desc'. Defaults to None.

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/locations'

        endpoint += f'?limit={limit}'

        if offset is not None:
            endpoint += f'&offset={offset}'
        if search is not None:
            endpoint += f'&search={search}'
        if sort is not None:
            endpoint += f'&sort={sort}'
        if order is not None:
            endpoint += f'&order={order}'
        
        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_location_by_id(self, id: int):
        """Get location by id

        Args:
            id (int): location id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/locations/{id}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def create(self, payload: dict):
        """Create location

        Args:
            payload (dict): Payload has to contain 'name'. Accepted:
                            # name (str),
                            # address (str, optional),
                            # address2 (str, optional),
                            # state (str, optional),
                            # country (str, optional),
                            # zip (str, optional),
                            # ldap_ou (str, optional),
                            # parent_id (int, optional),
                            # currency (str, optional),
                            # manager_id (int, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/locations'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def update(self, id: int, payload: dict):
        """Update location

        Args:
            id (int): location id
            payload (dict): Payload has to contain 'name'. Accepted:
                            # name (str),
                            # address (str, optional),
                            # address2 (str, optional),
                            # state (str, optional),
                            # country (str, optional),
                            # zip (str, optional),
                            # ldap_ou (str, optional),
                            # parent_id (int, optional),
                            # currency (str, optional),
                            # manager_id (int, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/locations/{id}'

        response = r.request('PUT', endpoint, headers=self.headers, json=payload)

        return response.json()

    def delete(self, id: int):
        """Delete location

        Args:
            id (int): location id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/locations/{id}'

        response = r.request('DELETE', endpoint, headers=self.headers)

        return response.json()



import requests as r

class Fields():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers

    def get(self):
        """Get custom fields

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/fields'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_field_by_id(self, id: int):
        """Get field by id

        Args:
            id (int): field id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/fields/{id}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def create(self, payload: dict):
        """Create field

        Args:
            payload (dict): Payload has to contain 'name', 'element'. Accepted:
                            # name (str)
                            # element (str) [text, textarea, checkbox, radio, listbox]
                            # field_values (str)
                            # show_in_email (bool)
                            # format (str)
                            # field_encrypted (bool)
                            # help_text (str)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/fields'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def update(self, id: int, payload: dict):
        """Update field

        Args:
            id (int): field ID
            Payload has to contain 'name', 'element'. Accepted:
                            # name (str)
                            # element (str) [text, textarea, checkbox, radio, listbox]
                            # field_values (str)
                            # show_in_email (bool)
                            # format (str)
                            # field_encrypted (bool)
                            # help_text (str)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/fields/{id}'

        response = r.request('PUT', endpoint, headers=self.headers, json=payload)

        return response.json()

    def associate_with(self, id: int, fieldset_id: int):
        """Associate custom field with a custom fieldset

        Args:
            id (int): field id
            fieldset_id (int): fieldset id

        Returns:
            dict: Json response
        """
        payload = {
            'fieldset_id': fieldset_id
        }

        endpoint = f'{self.server}/api/v1/fields/{id}/associate'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def disassociate_with(self, id: int, fieldset_id: int):
        """Disassociate custom field with a custom fieldset

        Args:
            id (int): field id
            fieldset_id (int): fieldset id

        Returns:
            dict: Json response
        """
        payload = {
            'fieldset_id': fieldset_id
        }

        endpoint = f'{self.server}/api/v1/fields/{id}/disassociate'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def delete(self, id: int):
        """Delete field

        Args:
            id (int): field id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/fields/{id}'

        response = r.request('DELETE', endpoint, headers=self.headers)

        return response.json()
    


import requests as r


class Assets():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers

    def get(self, limit: int=50, offset: int=None, search: str=None, order_number: str=None, sort: str=None, 
            order: str=None, model_id: int=None, category_id: int=None, manufacturer_id: int=None, company_id: int=None,
            location_id: int=None, status: str=None, status_id: int=None):
        """Return a list of assets

        Args:
            limit (int, optional): Number of entries to return. Defaults to 50.
            offset (int, optional): Offset to use. Defaults to None.
            search (str, optional): A text string to search the assets data for. Defaults to None.
            order_number (str, optional): Assets associated only with this order number. Defaults to None.
            sort (str, optional): Column to sort by. Defaults to None.
            order (str, optional): Order of sort 'asc' or 'desc'. Defaults to None.
            model_id (int, optional): Assets associated only with this model id. Defaults to None.
            category_id (int, optional): Assets associated only with this category id. Defaults to None.
            manufacturer_id (int, optional): Assets associated only with this manufacturer id. Defaults to None.
            company_id (int, optional): Assets associated only with this company id. Defaults to None.
            location_id (int, optional): Assets associated only with this location id. Defaults to None.
            status (str, optional): Assets associated only with this status. 'RTD', 'Deployed', 'Undeployable', 
                                    'Deleted', 'Archived', 'Requestable'. Defaults to None.
            status_id (int, optional): Assets associated only with this status id. Defaults to None.

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/hardware'

        endpoint += f'?limit={limit}'

        if offset is not None:
            endpoint += f'&offset={offset}'
        if search is not None:
            endpoint += f'&search={search}'
        if order_number is not None:
            endpoint += f'&order_number={order_number}'
        if sort is not None:
            endpoint += f'&sort={sort}'
        if order is not None:
            endpoint += f'&order={order}'
        if model_id is not None:
            endpoint += f'&model_id={model_id}'
        if category_id is not None:
            endpoint += f'&category_id={category_id}'
        if manufacturer_id is not None:
            endpoint += f'&manufacturer_id={manufacturer_id}'
        if company_id is not None:
            endpoint += f'&company_id={company_id}'
        if location_id is not None:
            endpoint += f'&location_id={location_id}'
        if status is not None:
            endpoint += f'&status={status}'
        if status_id is not None:
            endpoint += f'&status_id={status_id}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_asset_by_id(self, id: int):
        """Get asset by ID

        Args:
            id (int): ID of the asset to query

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/{id}'
        
        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_asset_by_asset_tag(self, asset_tag: str):
        """Get asset by asset tag

        Args:
            asset_tag (str): Asset tag

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/bytag/{asset_tag}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_asset_by_serial(self, serial: str):
        """Get asset by serial

        Args:
            serial (str): Serial

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/byserial/{serial}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_assets_due_for_audit(self):
        """Get assets due for audit

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/audit/due'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_assets_overdue_for_audit(self):
        """Get assets overdue for audit

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/audit/overdue'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_asset_licenses(self, id: int):
        """Get a list of licenses for asset

        Args:
            id (int): Asset id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/{id}/licenses'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def create(self, payload: dict):
        """Create a new asset

        Args:
            payload (dict): Payload has to contain 'asset_tag', 'status_id', 'model_id'. Accepted:
                            # asset_tag (str),
                            # status_id (str),
                            # model_id (int),
                            # name (str, optional),
                            # image (file, optional),
                            # serial (str, optional),
                            # purchase_date (str, optional),
                            # purchase_cost (float, optional),
                            # order_number (str, optional),
                            # notes (str, optional),
                            # archived (bool, optional),
                            # requestable (bool, optional),
                            # warranty_months (int, optional),
                            # depreciate (bool, optional),
                            # supplier_id (int, optional),
                            # rtd_location_id (int, optional),
                            # last_audit_date (str, optional),
                            # location_id (int, optional),

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def update(self, id: int, payload: dict):
        """Update asset

        Args:
            id (int): Asset id
            payload (dict): Payload has to contain 'asset_tag', 'status_id', 'model_id'. Accepted:
                            # asset_tag (str),
                            # status_id (str),
                            # model_id (int),
                            # name (str, optional),
                            # image (file, optional),
                            # serial (str, optional),
                            # purchase_date (str, optional),
                            # purchase_cost (float, optional),
                            # order_number (str, optional),
                            # notes (str, optional),
                            # archived (bool, optional),
                            # requestable (bool, optional),
                            # warranty_months (int, optional),
                            # depreciate (bool, optional),
                            # supplier_id (int, optional),
                            # rtd_location_id (int, optional),
                            # last_audit_date (str, optional),
                            # location_id (int, optional),
                            # last_checkout (str, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/{id}'

        response = r.request('PUT', endpoint, headers=self.headers, json=payload)

        return response.json()

    def delete(self, id: int):
        """Delete asset

        Args:
            id (int): Asset id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/{id}'

        response = r.request('DELETE', endpoint, headers=self.headers)

        return response.json()

    def checkout(self, id: int, payload: dict):
        """Checkout asset

        Args:
            id (int): Asset id
            payload (dict): Payload has to contain 'checkout_to_type'. Accepted:
                            # checkout_to_type (str) [user/asset/location],
                            # assigned_user (int, optional),
                            # assigned_asset (int, optional),
                            # assigned_location (int, optional),
                            # expected_checkin (str, optional),
                            # checkout_at (str, optional) [Override checkout date],
                            # name (str, optional) [New asset name],
                            # note (str, optional) 

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/{id}/checkout'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def checkin(self, id: int, payload: dict={}):
        """Checkout asset

        Args:
            id (int): Asset id
            payload (dict): Payload. Accepted:
                            # note (str, optional)
                            # location_id (int, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/hardware/{id}/checkin'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()



import requests as r

class Maintenances():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers

    def get(self, limit: int=50, offset: int=None, search: str=None, sort: str=None, order: str=None, asset_id: int=None):
        """Get maintenances

        Args:
            limit (int, optional): Number of entries to return. Defaults to 50.
            offset (int, optional): Offset to use. Defaults to None.
            search (str, optional): A text string to search the maintenances data for. Defaults to None.
            sort (str, optional): Column to sort by. Defaults to None.
            order (str, optional): Order of sort 'asc' or 'desc'. Defaults to None.
            asset_id (int, optional): Asset ID. Defaults to None

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/maintenances'

        endpoint += f'?limit={limit}'

        if offset is not None:
            endpoint += f'&offset={offset}'
        if search is not None:
            endpoint += f'&search={search}'
        if sort is not None:
            endpoint += f'&sort={sort}'
        if order is not None:
            endpoint += f'&order={order}'
        if asset_id is not None:
            endpoint += f'&asset_id={asset_id}'
        
        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def create(self, payload: dict):
        """Create maintenance

        Args:
            payload (dict): Payload has to contain 'title', 'asset_id', 'supplier_id', 'start_date'. Accepted:
                            # title (str)
                            # asset_id (str)
                            # supplier_id (str)
                            # is_warranty (bool, optional)
                            # cost (float, optional)
                            # notes (str, optional)
                            # asset_mainenance_type (str, optional) [Maintenance/Repair/PAT Test/Upgrade/Hardware Support/Software Support
                            # completion_date (str, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/maintenances'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def update(self, id: int, payload: dict):
        # Can't update?
        pass

    def delete(self, id: int):
        """Delete maintenance

        Args:
            id (int): maintenance id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/maintenances/{id}'

        response = r.request('DELETE', endpoint, headers=self.headers)

        return response.json()



import requests as r

class Models():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers

    def get(self, limit: int=50, offset: int=None, search: str=None, sort: str=None, order: str=None):
        """Get models

        Args:
            limit (int, optional): Number of entries to return. Defaults to 50.
            offset (int, optional): Offset to use. Defaults to None.
            search (str, optional): A text string to search the models data for. Defaults to None.
            sort (str, optional): Column to sort by. Defaults to None.
            order (str, optional): Order of sort 'asc' or 'desc'. Defaults to None.

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/models'

        endpoint += f'?limit={limit}'

        if offset is not None:
            endpoint += f'&offset={offset}'
        if search is not None:
            endpoint += f'&search={search}'
        if sort is not None:
            endpoint += f'&sort={sort}'
        if order is not None:
            endpoint += f'&order={order}'
        
        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_model_by_id(self, id: int):
        """Get model by id

        Args:
            id (int): model id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/models/{id}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def create(self, payload: dict):
        """Create model

        Args:
            payload (dict): Payload has to contain 'name', 'category_id', 'manufacturer_id'. Accepted:
                            # name (str)
                            # model_number (str, optional)
                            # category_id (int)
                            # manufacturer_id (int)
                            # eol (int, optional)
                            # fieldset_id (int, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/models'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def update(self, id: int, payload: dict):
        """Update model

        Args:
            id (int): model id
            payload (dict): Payload has to contain 'name', 'category_id', 'manufacturer_id'. Accepted:
                            # name (str)
                            # model_number (str, optional)
                            # category_id (int)
                            # manufacturer_id (int)
                            # eol (int, optional)
                            # fieldset_id (int, optional)
                            # deprecation_id (int)
                            # notes (str)
                            # requestable (bool)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/models/{id}'

        response = r.request('PUT', endpoint, headers=self.headers, json=payload)

        return response.json()

    def delete(self, id: int):
        """Delete model

        Args:
            id (int): model id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/models/{id}'

        response = r.request('DELETE', endpoint, headers=self.headers)

        return response.json()



import requests as r

class Departments():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers

    def get(self):
        """Get departments

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/departments'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_department_by_id(self, id: int):
        """Get department by id

        Args:
            id (int): Deparment id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/departments/{id}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def create(self, name: str):
        """Create department

        Args:
            name (str): Department name

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/departments'

        payload = {
            'name': name
        }

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def update(self, id: int, name: str):
        """Update department

        Args:
            id (int): Department ID
            name (str): Department name

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/departments/{id}'

        payload = {
            'name': name
        }

        response = r.request('PUT', endpoint, headers=self.headers, json=payload)

        return response.json()

    def delete(self, id: int):
        """Delete department

        Args:
            id (int): Department id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/departments/{id}'

        response = r.request('DELETE', endpoint, headers=self.headers)

        return response.json()
    


import requests as r

class Components():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers

    def get(self, limit: int=50, offset: int=None, search: str=None, sort: str=None, order: str=None, order_number: str=None, expand: str=None):
        """Get components

        Args:
            limit (int, optional): Number of entries to return. Defaults to 50.
            offset (int, optional): Offset to use. Defaults to None.
            search (str, optional): A text string to search the assets data for. Defaults to None.
            sort (str, optional): Column to sort by. Defaults to None.
            order (str, optional): Order of sort 'asc' or 'desc'. Defaults to None.
            order_number (str, optional): Components associated only with this order number. Defaults to None.
            expand (str, optional): [description]. Defaults to None.

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/components'

        endpoint += f'?limit={limit}'

        if offset is not None:
            endpoint += f'&offset={offset}'
        if search is not None:
            endpoint += f'&search={search}'
        if sort is not None:
            endpoint += f'&sort={sort}'
        if order is not None:
            endpoint += f'&order={order}'
        if order_number is not None:
            endpoint += f'&order_number={order_number}'
        if expand is not None:
            endpoint += f'&expand={expand}'
        
        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_component_by_id(self, id: int):
        """Get component by id

        Args:
            id (int): component id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/components/{id}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def assets(self, id: int):
        """Show which assets a components has been assigned to

        Args:
            id (int): component id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/components/{id}/assets'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def create(self, payload: dict):
        """Create component

        Args:
            payload (dict): Payload has to contain 'name', 'qty', 'category_id'. Accepted:
                            # name (str),
                            # qty (int),
                            # category_id (int),
                            # order_number (str, optional)
                            # purchase_cost (float, optional),
                            # purchase_date (str, optional),
                            # company_id (int, optional),
                            # location_id (int, optional),
                            # min_amt (int, optional),
                            # serial (str, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/components'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def update(self, id: int, payload: dict):
        """Update component

        Args:
            id (int): component id
            payload (dict): Payload has to contain 'name', 'qty', 'category_id'. Accepted:
                            # name (str),
                            # qty (int),
                            # category_id (int),
                            # order_number (str, optional)
                            # purchase_cost (float, optional),
                            # purchase_date (str, optional),
                            # company_id (int, optional),
                            # location_id (int, optional),
                            # min_amt (int, optional),
                            # serial (str, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/components/{id}'

        response = r.request('PUT', endpoint, headers=self.headers, json=payload)

        return response.json()

    def delete(self, id: int):
        """Delete component

        Args:
            id (int): component id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/components/{id}'

        response = r.request('DELETE', endpoint, headers=self.headers)

        return response.json()

    def checkout(self, id: int, payload: dict):
        """Checkout component

        Args:
            id (int): component id
            payload (dict): Payload has to contain 'assigned_to' and 'assigned_qty'. Accepted:
                            # assigned_to (int) [user],
                            # assigned_qty (int)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/components/{id}/checkout'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def checkin(self, id: int, checkin_qty: int):
        """Checkin component

        Args:
            id (int): component id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/components/{id}/checkin'

        payload = {
            'checkin_qty': checkin_qty
        }

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()



from Accessories import Accessories
from Assets import Assets
from Categories import Categories
from Companies import Companies
from Components import Components
from Consumables import Consumables
from .Departments import Departments
from .Fields import Fields
from .Licenses import Licenses
from .Locations import Locations
from .Maintenances import Maintenances
from .Manufacturers import Manufacturers
from .Models import Models
from .Reports import Reports
from .StatusLabels import StatusLabels
from .Users import Users

class SnipeIT():
    def __init__(self, server, token):
        self.server = server
        self.token = token
        self.headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + self.token
        }

        self.accessories = Accessories(self.server, self.headers)
        self.assets = Assets(self.server, self.headers)
        self.categories = Categories(self.server, self.headers)
        self.companies = Companies(self.server, self.headers)
        self.components = Components(self.server, self.headers)
        self.consumables = Consumables(self.server, self.headers)
        self.departments = Departments(self.server, self.headers)
        self.fields = Fields(self.server, self.headers)
        self.licenses = Licenses(self.server, self.headers)
        self.locations = Locations(self.server, self.headers)
        self.maintenances = Maintenances(self.server, self.headers)
        self.manufacturers = Manufacturers(self.server, self.headers)
        self.models = Models(self.server, self.headers)
        self.reports = Reports(self.server, self.headers)
        self.status_labels = StatusLabels(self.server, self.headers)
        self.users = Users(self.server, self.headers)
        


import requests as r

class Reports():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers

    def get(self, limit: int=50, offset: int=None, search: str=None, target_type: str=None, target_id: int=None,
            item_type: str=None, item_id: int=None, action_type: str=None):
        """Get consumables

        Args:
            limit (int, optional): Number of entries to return. Defaults to 50.
            offset (int, optional): Offset to use. Defaults to None.
            search (str, optional): A text string to search the consumables data for. Defaults to None.
            target_type (str, optional): Type of traget checked out to (App/Model/User). Defaults to None.
            target_id (int, optional): ID of target. Defaults to None.
            item_type (str, optiona): Type of item (App/Model/Asset). Defaults to None.
            item_id (int, optional): ID of item. Defaults to None.
            action_type (str, optional): Type of action (checkin from, checkout, update, add seats, etc.). Defaults to None

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/consumables'

        endpoint += f'?limit={limit}'

        if offset is not None:
            endpoint += f'&offset={offset}'
        if search is not None:
            endpoint += f'&search={search}'
        if target_type is not None:
            endpoint += f'&target_type={target_type}'
        if target_id is not None:
            endpoint += f'&target_id={target_id}'
        if item_type is not None:
            endpoint += f'&item_type={item_type}'
        if item_id is not None:
            endpoint += f'&item_id={item_id}'
        if action_type is not None:
            endpoint += f'&action_type={action_type}'
        
        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()


import requests as r

class Users():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers
    
    def get(self, limit: int=50, offset: int=None, search: str=None, sort: str=None, order: str=None, first_name: str=None,
            last_name: str=None, username: str=None, email: str=None, employee_num: str=None, state: str=None, zip: str=None,
            country: str=None, group_id: int=None, department_id: int=None, company_id: int=None, location_id: int=None,
            deleted: bool=None, all: bool=None):
        """Get users

        Args:
            limit (int, optional): Number of entries to return. Defaults to 50.
            offset (int, optional): Offset to use. Defaults to None.
            search (str, optional): A text string to search the users data for. Defaults to None.
            sort (str, optional): Column to sort by. Defaults to None.
            order (str, optional): Order of sort 'asc' or 'desc'. Defaults to None.
            first_name (str, optional): First name. Defaults to None.
            last_name (str, optional): Last name. Defaults to None.
            username (str, optional): Username. Defaults to None.
            email (str, optional): Email. Defaults to None.
            employee_num (str, optional): Employee number. Defaults to None.
            state (str, optional): State. Defaults to None.
            zip (str, optional): Zip. Defaults to None.
            country (str, optional): Country. Defaults to None.
            group_id (int, optional): Group id. Defaults to None.
            department_id (int, optional): Department id. Defaults to None.
            company_id (int, optional): company id. Defaults to None.
            location_id (int, optional): locaton id. Defaults to None.
            deleted (bool, optional): Deleted user. Defaults to None.
            all (bool, optional): Both deleted and active. Defaults to None.

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/users'

        endpoint += f'?limit={limit}'

        if offset is not None:
            endpoint += f'&offset={offset}'
        if search is not None:
            endpoint += f'&search={search}'
        if sort is not None:
            endpoint += f'&sort={sort}'
        if order is not None:
            endpoint += f'&order={order}'
        if first_name is not None:
            endpoint += f'&first_name={first_name}'
        if last_name is not None:
            endpoint += f'&last_name={last_name}'
        if username is not None:
            endpoint += f'&username={username}'
        if email is not None:
            endpoint += f'&email={email}'
        if employee_num is not None:
            endpoint += f'&employee_num={employee_num}'
        if state is not None:
            endpoint += f'&state={state}'
        if zip is not None:
            endpoint += f'&zip={zip}'
        if country is not None:
            endpoint += f'&country={country}'
        if group_id is not None:
            endpoint += f'&group_id={group_id}'
        if department_id is not None:
            endpoint += f'&department_id={department_id}'
        if company_id is not None:
            endpoint += f'&company_id={company_id}'
        if location_id is not None:
            endpoint += f'&location_id={location_id}'
        if deleted is not None:
            endpoint += f'&deleted={deleted}'
        if all is not None:
            endpoint += f'&all={all}'
        
        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_user_by_id(self, id: int):
        """Get user by id

        Args:
            id (int): user id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/users/{id}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_user_assets(self, id: int):
        """Get user's assets

        Args:
            id (int): user id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/users/{id}/assets'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_user_accessories(self, id: int):
        """Get user's accessories

        Args:
            id (int): user id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/users/{id}/accessories'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_user_licenses(self, id: int):
        """Get user's licenses

        Args:
            id (int): user id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/users/{id}/licenses'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def me(self):
        """Get own user

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/users/me'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def create(self, payload: dict):
        """Create user

        Args:
            payload (dict): Payload has to contain 'first_name', 'username', 'password', 'password_confirmation'. Accepted:
                            # first_name (str),
                            # last_name (str, optional)
                            # username (str, optional),
                            # password (str, optional),
                            # password_confirmation (str, optional),
                            # email (str, optional),
                            # permissions (str, optional),
                            # activated (bool, optional),
                            # phone (str, optional),
                            # jobtitle (str, optional),
                            # manager_id (int, optional),
                            # employee_num (str, optional),
                            # notes (str, optional),
                            # company_id (int, optional),
                            # two_factor_enrolled (bool, optional),
                            # two_factor_optin (bool, optional),
                            # department_id (int, optional),
                            # location_id (int, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/users'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def update(self, id: int, payload: dict):
        """Update user

        Args:
            id (int): user id
            payload (dict): Payload. Accepted:
                            # first_name (str),
                            # last_name (str, optional)
                            # username (str, optional),
                            # password (str, optional),
                            # email (str, optional),
                            # permissions (str, optional),
                            # activated (bool, optional),
                            # phone (str, optional),
                            # jobtitle (str, optional),
                            # manager_id (int, optional),
                            # employee_num (str, optional),
                            # notes (str, optional),
                            # company_id (int, optional),
                            # two_factor_enrolled (bool, optional),
                            # two_factor_optin (bool, optional),
                            # department_id (int, optional),
                            # location_id (int, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/users/{id}'

        response = r.request('PUT', endpoint, headers=self.headers, json=payload)

        return response.json()

    def delete(self, id: int):
        """Delete user

        Args:
            id (int): user id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/users/{id}'

        response = r.request('DELETE', endpoint, headers=self.headers)

        return response.json()



import requests as r

class Consumables():
    def __init__(self, server, headers):
        self.server = server
        self.headers = headers

    def get(self, limit: int=50, offset: int=None, search: str=None, sort: str=None, order: str=None, order_number: str=None, 
            expand: str=None, category_id: int=None, company_id: int=None, manufacturer_id: int=None):
        """Get consumables

        Args:
            limit (int, optional): Number of entries to return. Defaults to 50.
            offset (int, optional): Offset to use. Defaults to None.
            search (str, optional): A text string to search the consumables data for. Defaults to None.
            sort (str, optional): Column to sort by. Defaults to None.
            order (str, optional): Order of sort 'asc' or 'desc'. Defaults to None.
            order_number (str, optional): consumables associated only with this order number. Defaults to None.
            expand (str, optional): Show additional details. Defaults to None.
            category_id (int, optional): consumables associated only with this category id. Defaults to None.
            manufacturer_id (int, optional): consumables associated only with this manufacturer id. Defaults to None.
            company_id (int, optional): consumables associated only with this company id. Defaults to None.

        Returns:
            dict: Json response
        """
        endpoint = self.server + '/api/v1/consumables'

        endpoint += f'?limit={limit}'

        if offset is not None:
            endpoint += f'&offset={offset}'
        if search is not None:
            endpoint += f'&search={search}'
        if sort is not None:
            endpoint += f'&sort={sort}'
        if order is not None:
            endpoint += f'&order={order}'
        if order_number is not None:
            endpoint += f'&order_number={order_number}'
        if expand is not None:
            endpoint += f'&expand={expand}'
        if category_id is not None:
            endpoint += f'&category_id={category_id}'
        if company_id is not None:
            endpoint += f'&company_id={company_id}'
        if manufacturer_id is not None:
            endpoint += f'&manufacturer_id={manufacturer_id}'
        
        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def get_consumable_by_id(self, id: int):
        """Get consumable by id

        Args:
            id (int): consumable id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/consumables/{id}'

        response = r.request('GET', endpoint, headers=self.headers)

        return response.json()

    def create(self, payload: dict):
        """Create consumable

        Args:
            payload (dict): Payload has to contain 'name', 'qty', 'category_id'. Accepted:
                            # name (str),
                            # qty (int),
                            # category_id (int),
                            # order_number (str, optional)
                            # purchase_cost (float, optional),
                            # purchase_date (str, optional),
                            # model_number (int, optional),
                            # company_id (int, optional),
                            # location_id (int, optional),
                            # manufacturer_id (int, optional),
                            # supplier_id (int, optional),
                            # min_amt (int, optional),
                            # item_no (str, optional)
                            # requestable (bool, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/consumables'

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def update(self, id: int, payload: dict):
        """Update consumable

        Args:
            id (int): consumable id
            payload (dict): Payload has to contain 'name', 'qty', 'category_id'. Accepted:
                            # name (str),
                            # qty (int),
                            # category_id (int),
                            # order_number (str, optional)
                            # purchase_cost (float, optional),
                            # purchase_date (str, optional),
                            # model_number (int, optional),
                            # company_id (int, optional),
                            # location_id (int, optional),
                            # manufacturer_id (int, optional),
                            # supplier_id (int, optional),
                            # min_amt (int, optional),
                            # item_no (str, optional)
                            # requestable (bool, optional)

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/consumables/{id}'

        response = r.request('PUT', endpoint, headers=self.headers, json=payload)

        return response.json()

    def delete(self, id: int):
        """Delete consumable

        Args:
            id (int): consumable id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/consumables/{id}'

        response = r.request('DELETE', endpoint, headers=self.headers)

        return response.json()

    def checkout(self, id: int, assigned_to: int=None):
        """Checkout consumable

        Args:
            id (int): consumable id
            assigned_to (int, optional): Assigned to person ID

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/consumables/{id}/checkout'

        payload = {
            'assigned_to': assigned_to
        }

        response = r.request('POST', endpoint, headers=self.headers, json=payload)

        return response.json()

    def checkin(self, id: int):
        """Checkin consumable

        Args:
            id (int): consumable id

        Returns:
            dict: Json response
        """
        endpoint = f'{self.server}/api/v1/consumables/{id}/checkin'

        response = r.request('POST', endpoint, headers=self.headers)

        return response.json()
